package cn.com.countrygarden.bi.main.service.performance;

import cn.com.countrygarden.bi.main.model.performance.UserTarget;

import java.util.List;
import java.util.Map;

public interface UserTargetService {
    int deleteByPrimaryKey(Integer utid);

    int insertSelective(UserTarget record);

    UserTarget selectByPrimaryKey(Integer utid);

    int updateByPrimaryKeySelective(UserTarget record);

    UserTarget selectCouldPower(UserTarget record);

    /**
     * 查询所有的userCode数据
     *
     * @param map 分页
     * @return 放回所有数据List
     */
    List<String> selectAllUserCode(Map<String, Integer> map);

    /**
     * 删除同一个userCode的所有数据
     *
     * @param userCode userCode
     * @return 删除结果
     */
    int deleteUserCodeAll(String userCode);

    /**
     * 验证是否存在userCode
     *
     * @param userCode userCode
     * @return Long
     */
    Long selectExistUser(String userCode);

    Long countSelectAllUserCode();

    /**
     * 根据userCode更新删除状态
     *
     * @param userCode UserCode
     * @return 更新的条数
     */
    int updateIsDelete(String userCode);
}
